#include <assert.h>
#include <ctype.h>
#include <err.h>

#include "libcreddy_common.h"

void *creddy_xmalloc(size_t len) {
    void *ret = malloc(len);
    if (ret == NULL)
        err(1, "couldn't malloc %d bytes\n", len);
    return ret;
}

void *creddy_xrealloc(void *ptr, size_t size) {
    void *ret = realloc(ptr, size);
    if (ret == NULL)
        err(1, "couldn't realloc %d bytes\n", size);
    return ret;
}

char *creddy_xstrdup(char *string) {
    char *dup = strdup(string);
    if (dup == NULL)
        err(1, "Can't dup %s", string);
    return dup;
}

int creddy_clean_name(char *string) {
    int i;

    assert(string != NULL);

    // Name must be alphanumeric or _
    for (i = 0; string[i] != '\0'; ++i)
        if (!isalnum(string[i]) && string[i] != '_')
            return 0;

    return 1;
}

chunk_t creddy_generate_serial() {
    chunk_t serial = chunk_empty;

    // create a serial (stolen from strongswan pki)
    rng_t *rng = lib->crypto->create_rng(lib->crypto, RNG_WEAK);
    if (!rng)
        errx(1, "no random number generator");

    rng->allocate_bytes(rng, 8, &serial);
    while (serial.ptr[0] == 0)
        // don't get leading 0's
        rng->get_bytes(rng, 1, serial.ptr);
    rng->destroy(rng);

    return serial;
}
